#include  "compile.hxx"

static char copyright1[] = "(C)Board of Trustees of the University of Illinois, 1992";
static char copyright2[] = "Program conceived by Jeremy Atack and Fred Bateman";
static char copyright3[] = "Developed by Mary Eschelbach Gregson";
static char copyright4[] = "Programmed by Timothy Gregson";
static char copyright5[] = "Funded by the National Science Foundation under grants SES-8914861 and SES-9001066";

struct
{
  char  *string;
  int   type;
  int   numflds;
} block_tab[] = {
                  {"NYSIIS", 1, 4},           // creates output field for nysiis code in block file
                  {"SOUNDEX", 2, 4},          // creates output field for soundex code in block file
                  {"COPY", 3, 2},             // copies this field to output field in block file
                  {"FIRSTXLETTERS", 4, 5},    // copies first X letters of field to output field in block file
                  {"", 0, 0}
                };

struct
{
  char  *string;
  int   type;
  int   numflds;
} filter_tab[] =  {
                    {"NAME", 1, 3},
                    {"NUMERIC", 2, 3}
                  };

struct
{
  char  *string;
  int   type;
  int   numflds;
  char  multi;
} match_tab[] = {
                  {"NAME", 1, 2, 1},                // name match, followed by sub match types
                  {"NUMERIC", 2, -1, 0},            // numeric match, either exact or with ranges
                  {"TEXT", 3, 2, 0},                // exact text match field
                  {"", 0, 0, 0}
                };

struct
{
  char  *string;
  int   stype;
  int   numflds;
} sub_match_tab[] = {
                      {"EXACT", 10, 0},              // name matches exact
                      {"FIRSTXLETTERS", 11, 1},      // name matches on first X letters
//                      {"GUTH", 12, 0},               // name agrees by GUTH algorith
                      {"BLOCK", 13, 0},             // block codes match exact
                      {"NYSIIS", 13, 0},             // alias name for above
                      {"SOUNDEX", 14, 0},            // soundex codes match exact
//                      {"INC", 15, 0},                // this one doesn't work yet - match by INClusion algorithm
                      {"", 0, 0}
                    };

char    instr[255];                         // a temp string var

void    main (int argc, char *argv[])
{
  FILE    *in, *out;                          // file pointers
  int     select = 0;                         // number of selections
  int     block = 0;                          // number of blockings
  int     filter = 0;                         // number of filters
  int     match = 0;                          // number of matchings
  long    inpos = 0L;                         // input file position
  long    temppos = 0L;
  int     dbhanda = 0;                        // file a dbase handle
  int     dbhandb = 0;                        // file b dbase handle

  dbinit ();
  if (argc == 4)
  {

    if ((dbhanda = dbopen (argv[1], 0)) < 0)
    {
      fprintf (stderr, "Couldn't open dbase file: %s\n", argv[1]);
      exit (1);
    }
    if ((dbhandb = dbopen (argv[2], 0)) < 0)
    {
      fprintf (stderr, "Couldn't open dbase file: %s\n", argv[2]);
      exit (2);
    }
    if ((in = fopen (argv[3], "rb")) == NULL)
    {
      fprintf (stderr, "Couldn't open input file: %s\n", argv[3]);
      exit (3);
    }
    strcpy (&argv[3][strlen(argv[3]) - 3], "CMP");
    if ((out = fopen (argv[3], "wb")) == NULL)
    {
      fprintf (stderr, "Couldn't create output file: %s\n", argv[3]);
      exit (4);
    }
  }
  else
  {
    fprintf (stderr, "Usage: compile DBFilea DBFileb scrfile\n");
    exit (-1);
  }
  fwrite (&select, 1, sizeof(int), out);           // save space for counters
  fwrite (&block, 1, sizeof(int), out);            // in output file
  fwrite (&filter, 1, sizeof(int), out);
  fwrite (&match, 1, sizeof(int), out);
  while (!feof (in))
  {
    fseek (in, inpos, SEEK_SET);
    get_token (in, instr, &inpos, 1);
    skip_line ( in, &inpos);
    if (strcmp (instr, "SELECT") == 0)
    {
      if (block != 0 || match != 0)
      {
        fprintf (stderr, "Selection section must come before BLOCK & MATCH\n");
        exit (-5);
      }
      else if (select != 0)
      {
        fprintf (stderr, "Only one selection section allowed\n");
        exit (-6);
      }
      else
        proc_select (in, out, &select, dbhanda, dbhandb);
    }
    else if (strcmp (instr, "BLOCK") == 0)
    {
      if (match != 0)
      {
        fprintf (stderr, "Blocking section must come before MATCH\n");
        exit (-5);
      }
      else if (block != 0)
      {
        fprintf (stderr, "Only one blocking section allowed\n");
        exit (-6);
      }
      else
        proc_block (in, out, &block, dbhanda, dbhandb);
    }
    else if (strcmp (instr, "FILTER") == 0)
    {
      if (match != 0)
      {
        fprintf (stderr, "Filter section must come before MATCH\n");
        exit (-5);
      }
      else if (filter != 0)
      {
        fprintf (stderr, "Only one filter section allowed\n");
        exit (-6);
      }
      else
        proc_filter (in, out, &filter, dbhanda, dbhandb);
    }
    else if (strcmp (instr, "MATCH") == 0)
    {
      if (match == 0)
        proc_match (in, out, &match, dbhanda, dbhandb);
      else
      {
        fprintf (stderr, "Only one matching section allowed\n");
        exit (-6);
      }
    }
    else
    {
      fprintf (stderr, "Unknown keyword found: %s\n", instr);
      exit (-7);
    }
  }
  fseek (out, 0L, SEEK_SET);                  // reposition back to beginning
  fwrite (&select, 1, sizeof(int), out);      // update field counts
  fwrite (&block, 1, sizeof(int), out);
  fwrite (&filter, 1, sizeof(int), out);
  fwrite (&match, 1, sizeof(int), out);
  fclose (in);
  fclose (out);
  dbclose (dbhanda);
  dbclose (dbhandb);
}

void  proc_select (FILE *in, FILE *out, int *count, int dbhanda, int dbhandb)
{
  char  *ptr;
  static  char  fld1[11], fld2[11], fld3[64], fld4[64];
  long  inpos;
  static  struct select_out sel;

  while (!get_token (in, fld1, &inpos, 0))        // loop until keyword found
  {
    memset (&sel, 0x0, sizeof (struct select_out));
    get_line (in, instr, &inpos, 254);            // get rest of line
    ptr = instr;
    ptr = get_fld (ptr, fld2, 10);                // get second field
    ptr = get_fld (ptr, fld3, 63);                // get third field
    ptr = get_fld (ptr, fld4, 63);                // get fourth field
    if (strcmp (fld1, "NULL") != 0)
    {
      if ((sel.fldnuma = dbgetfnum (dbhanda, fld1)) == 0)
      {
        fprintf (stderr, "Unknown field name on selection record: %s\n", fld1);
        exit (-10);
      }
    }
    if (strcmp (fld2, "NULL") != 0)
    {
      if ((sel.fldnumb = dbgetfnum (dbhandb, fld2)) == 0)
      {
        fprintf (stderr, "Unknown field name on selection record: %s\n", fld2);
        exit (-10);
      }
    }
    strcpy (sel.string1, fld3);
    strcpy (sel.string2, fld4);
    fwrite (&sel, 1, sizeof (struct select_out), out);
    (*count)++;
  }
}

void  proc_block (FILE *in, FILE *out, int *count, int dbhanda, int dbhandb)
{
  char  *ptr;
  static  char  btype[255];
  static  char  fld1[11];
  static  char  fld2[11];
  static  char  fld3[11];
  static  char  fld4[11];
  static  char  fld5[6];
  long  inpos;
  static  struct  block_out bl;
  int   i;

  while (!get_token (in, btype, &inpos, 0))
  {
    memset (&bl, 0x0, sizeof (struct block_out));
    get_line (in, instr, &inpos, 254);
    ptr = instr;
    ptr = get_fld (ptr, fld1, 10);
    ptr = get_fld (ptr, fld2, 10);
    ptr = get_fld (ptr, fld3, 10);
    ptr = get_fld (ptr, fld4, 10);
    ptr = get_fld (ptr, fld5, 5);
    for (i = 0;block_tab[i].type && strcmp (btype, block_tab[i].string) != 0;i++)
    {
      // empty loop to find block type in table
    }
    if (!block_tab[i].type)
    {
      fprintf (stderr, "Unknown blocking type: %s\n", btype);
      exit (-20);
    }
    bl.btype = block_tab[i].type;
    if (block_tab[i].numflds > 0)
    {
      if ((bl.fldnuma = dbgetfnum (dbhanda, fld1)) == 0)
      {
        fprintf (stderr, "Unknown field name on block record: %s\n", fld1);
        exit (-21);
      }
    }
    if (block_tab[i].numflds > 1)
    {
      if ((bl.fldnumb = dbgetfnum (dbhandb, fld2)) == 0)
      {
        fprintf (stderr, "Unknown field name on block record: %s\n", fld2);
        exit (-21);
      }
    }
    if (block_tab[i].numflds > 2)
    {
      if (fld3[0] == 0)
      {
        fprintf (stderr, "Blocking type requires a third parameter: %s\n", btype);
        exit (-22);
      }
      strcpy (bl.fldnamea, fld3);
    }
    if (block_tab[i].numflds > 3)
    {
      if (fld4[0] == 0)
      {
        fprintf (stderr, "Blocking type requires a fourth parameter: %s\n", btype);
        exit (-22);
      }
      strcpy (bl.fldnameb, fld4);
    }
    if (block_tab[i].numflds > 4)
    {
      if (fld5[0] == 0)
      {
        fprintf (stderr, "Blocking type requires a fifth parameter: %s\n", btype);
        exit (-22);
      }
      bl.len = atoi (fld5);
    }
    if (bl.btype == 3)
    {
      strcpy (bl.fldnamea, fld1);
      strcpy (bl.fldnameb, fld2);
    }
    fwrite (&bl, 1, sizeof (struct block_out), out);
    (*count)++;
  }
}

void  proc_filter (FILE *in, FILE *out, int *count, int dbhanda, int dbhandb)
{
  char  *ptr;
  static  char  ftype[255];
  static  char  fld1[11];
  static  char  fld2[11];
  static  char  fld3[11];
  static  struct  filter_out  fil;
  long  inpos;
  int   i;

  while (!get_token (in, ftype, &inpos, 0))
  {
    memset (&fil, 0x0, sizeof (struct filter_out));
    get_line (in, instr, &inpos, 254);
    ptr = instr;
    ptr = get_fld (ptr, fld1, 10);
    ptr = get_fld (ptr, fld2, 10);
    ptr = get_fld (ptr, fld3, 10);
    for (i = 0;filter_tab[i].type && strcmp (ftype, filter_tab[i].string) != 0;i++)
    {
      // empty loop to find filter type in table
    }
    if (!filter_tab[i].type)
    {
      fprintf (stderr, "Unknown filter type: %s\n", ftype);
      exit (-30);
    }
    fil.ftype = filter_tab[i].type;
    if ((fil.fldnuma = dbgetfnum (dbhanda, fld1)) == 0)
    {
      fprintf (stderr, "Unknown field name on filter record: %s\n", fld1);
      exit (-31);
    }
    if ((fil.fldnumb = dbgetfnum (dbhandb, fld2)) == 0)
    {
      fprintf (stderr, "Unknown field name on filter record: %s\n", fld2);
      exit (-31);
    }
    fil.count1 = atoi (fld3);
    fwrite (&fil, 1, sizeof (struct filter_out), out);
    (*count)++;
  }
}

void  proc_match (FILE *in, FILE *out, int *count, int dbhanda, int dbhandb)
{
  char  *ptr;
  char  *item1;
  char  *item2;
  char  *tweight;
  char  mtype[40];
  char  fld1[11];
  char  fld2[11];
  char  fldn[20];
  long  inpos;
  struct  match_out mat;
  int   i;
  double  tmp;

  while (!get_token (in, mtype, &inpos, 0))
  {
    memset (&mat, 0x0, sizeof (struct match_out));
    get_line (in, instr, &inpos, 254);
    ptr = instr;
    ptr = get_fld (ptr, fld1, 10);
    ptr = get_fld (ptr, fld2, 10);
    for (i = 0;match_tab[i].type && strcmp (mtype, match_tab[i].string) != 0;i++)
    {
      // empty loop to find match type in table
    }
    if (!block_tab[i].type)
    {
      fprintf (stderr, "Unknown matching type: %s\n", mtype);
      exit (-30);
    }
    mat.mtype = match_tab[i].type;
    mat.level = 1;
    mat.data1 = 0;
    mat.data2 = 0;
    if ((mat.fldnuma = dbgetfnum (dbhanda, fld1)) == 0)
    {
      fprintf (stderr, "Unknown field name on match record: %s\n", fld1);
      exit (-31);
    }
    if ((mat.fldnumb = dbgetfnum (dbhandb, fld2)) == 0)
    {
      fprintf (stderr, "Unknown field name on match record: %s\n", fld2);
      exit (-31);
    }
    if (match_tab[i].multi)
    {
      proc_multi_match (in, out, count, dbhanda, dbhandb, mat.fldnuma, mat.fldnumb);
    }
    else
    {
      fwrite (&mat, 1, sizeof (struct match_out), out);
      (*count)++;
    }
    if (match_tab[i].numflds == -1)
    {
      item1 = NULL;
      item2 = NULL;
      tweight = NULL;
      for (ptr = get_fld (ptr, fldn, 19) ; fldn[0] != 0; ptr = get_fld (ptr, fldn, 19))
      {
        if (isdigit (fldn[0]))
          item1 = fldn;
        if ((item2 = strchr (fldn, '/')) != NULL)
          (*item2++) = 0;
        if ((tweight = strchr (fldn, '|')) != NULL)
          (*tweight++) = 0;
        if (item1 != NULL)
          mat.data1 = atoi (item1);
        if (item2 != NULL)
          mat.data2 = atoi (item2);
        else
          mat.data2 = mat.data1;
        if (item1 != NULL)
        {
          mat.level++;
          fwrite (&mat, 1, sizeof (struct match_out), out);
          (*count)++;
        }
        if (tweight != NULL)
          tmp = atof (tweight);
        else
          tmp = 0.0;
        item1 = NULL;
        item2 = NULL;
        tweight = NULL;
      }
    }
  }
}

void  proc_multi_match (FILE *in, FILE *out, int *count, int dbhanda, int dbhandb, int fldnuma, int fldnumb)
{
  char  stype[40];
  char  fld[11];
  char  *ptr;
  long  inpos;
  long  oinpos;
  struct  match_out mat;
  int   token;
  int   i;

  memset (&mat, 0x0, sizeof (struct match_out));
  mat.fldnuma = fldnuma;
  mat.fldnumb = fldnumb;
  do
  {
    oinpos = inpos;
    token = get_token (in, stype, &inpos, 0);
    if (!token)
    {
      for (i = 0;sub_match_tab[i].stype && strcmp (stype, sub_match_tab[i].string) != 0;i++)
      {
        // empty loop to find sub match type in table
      }
      if (sub_match_tab[i].stype)
      {
        mat.mtype = sub_match_tab[i].stype;
        mat.level++;
        mat.data1 = 0;
        mat.data2 = 0;
        get_line (in, instr, &inpos, 254);
        ptr = instr;
        if (sub_match_tab[i].numflds == 1)          // get count field
        {
          ptr = get_fld (ptr, fld, 10);
          mat.data1 = atoi (fld);
        }
        fwrite (&mat, 1, sizeof (struct match_out), out);
        (*count)++;
      }
    }
  } while (!token && sub_match_tab[i].stype);
  fseek (in, oinpos, SEEK_SET);
}

char  *get_fld (char *instr, char *outstr, int count)
{
  while (*instr != 0 && (*instr == 0x20 || *instr == 0x09))
    instr++;                                    // skip over white space
  while (*instr != 0 && *instr != 0x20 && *instr != 0x09)
  {
    if (count)                                  // only process while field space
    {
      *outstr = *instr;
      outstr++;
      count--;
    }
    instr++;
  }
  *outstr = 0;
  return (instr);
}

int   get_token (FILE *fp, char *str, unsigned long *filepos, int kw_flag)
{
  int   beg_line = false;              //true = token at beg. of line
  int   routine_start = true;
  char  ch;

  do
  {
    ch = (char)getc (fp);
    *filepos = (unsigned long)ftell(fp);
    if (*filepos == 1L)
      beg_line = true;
    if (ch == 0x20 || ch == 0x09)
    {
      if (!routine_start)
      {
        (void)ungetc (ch, fp);
        (*filepos)--;
        return (beg_line);
      }
      else
        beg_line = false;
    }
    else if (ch == 0x0a || ch == 0x0d)
    {
      if (routine_start)
      {
        beg_line = true;
      }
      else
      {
        (void)ungetc (ch, fp);
        (*filepos)--;
        return (beg_line);
      }
    }
    else
    {
      if ((kw_flag && beg_line) || !kw_flag)
      {
        routine_start = false;
        *str++ = ch;
        *str = 0;                       // insert new NULL
      }
    }
  } while (!feof (fp));
  return (beg_line);
}

void  get_line (FILE *fp, char *str, unsigned long *filepos, int max_count)
{
  char  ch;
  int   curr_len = 0;
  int   routine_start = true;

  do
  {
    ch = (char)getc (fp);
    *filepos = (unsigned long)ftell(fp);
    if ((ch == 0x20 || ch == 0x09) && routine_start)
    {
      // empty loop to eat up leading whitespace
    }
    else if (ch != 0x0d && ch != 0x0a)
    {
      routine_start = false;
      if (curr_len < max_count)
      {
        *str++ = ch;
        curr_len++;
      }
    }
  } while (ch != 0x0d && ch != 0x0a);
  (void)ungetc (ch, fp);
  (*filepos)--;
  *str = 0;
}

void  skip_line (FILE *fp, unsigned long *filepos)
{
  char  ch;

  do
  {
    ch = (char)getc (fp);
    *filepos = (unsigned long)ftell(fp);
  } while (ch != 0x0d && ch != 0x0a);
  (void)ungetc (ch, fp);
  (*filepos)--;
}

void  pascal  freemem (void)
{
  // dummy routine for dblib to link correctly
}
