#include  <stdio.h>
#include  <string.h>
#include  <stdlib.h>

#include  "dblib.h"

static char copyright1[] = "(C)Board of Trustees of the University of Illinois, 1992";
static char copyright2[] = "Program conceived by Jeremy Atack and Fred Bateman";
static char copyright3[] = "Developed by Mary Eschelbach Gregson";
static char copyright4[] = "Programmed by Timothy Gregson";
static char copyright5[] = "Funded by the National Science Foundation under grants SES-8914861 and SES-9001066";

void  main (int argc, char **argv);

void  main (int argc, char **argv)
{
//  DB                  *lnk, *filea, *fileb;
  int                 lnk, filea, fileb;
  struct  file_info   data_a, data_b;
  char                buf[50], buf1[50], buf2[50];
  long                recnuma;
  long                recnumb;
  int                 grpflda = 0, grpfldb = 0, lnkflda, lnkfldb;
  long                sequence = 0;

  dbinit ();
  if (argc < 6 || argc > 8)
  {
    fprintf (stderr, "Usage: linkit filea fileb deffile lnkflda lnkfldb <grpflda|NONE> <grpfldb|NONE>\n");
    exit (1);
  }
  if ((filea = dbopen (argv[1], 1)) < 0)
  {
    fprintf (stderr, "Unable to open file a: %s\n", argv[1]);
    exit (2);
  }
  if ((fileb = dbopen (argv[2], 1)) < 0)
  {
    fprintf (stderr, "Unable to open file b: %s\n", argv[2]);
    exit (3);
  }
  dbgetinfo (filea, &data_a);
  dbgetinfo (fileb, &data_b);
  if ((lnkflda = dbgetfnum (filea, argv[4])) == 0)
  {
    fprintf (stderr, "Unknown link field A name: %s\n", argv[4]);
    exit (7);
  }
  if ((lnkfldb = dbgetfnum (fileb, argv[5])) == 0)
  {
    fprintf (stderr, "Unknown link field B name: %s\n", argv[5]);
    exit (8);
  }
  if ((lnk = dbopen (argv[3], 0)) < 0)
  {
    fprintf (stderr, "Unable to open definite link file: %s\n", argv[3]);
    exit (4);
  }
  if (argc > 6)
  {
    if (strcmp (strupr(argv[6]), "NONE") != 0)
      if ((grpflda = dbgetfnum (filea, argv[6])) == 0)
      {
        fprintf (stderr, "Unknown group field A name: %s\n", argv[6]);
        exit (5);
      }
  }
  if (argc > 7)
  {
    if (strcmp (strupr(argv[7]), "NONE") != 0)
      if ((grpfldb = dbgetfnum (fileb, argv[7])) == 0)
      {
        fprintf (stderr, "Unknown group field B name: %s\n", argv[7]);
        exit (6);
      }
  }
  while (!dbread(lnk))
  {
    sequence++;   // increment linkage number
    sprintf (buf, "%ld", sequence);
    dbgetfld (lnk, 1, buf1);
    recnuma = atol(buf1);
    dbgetfld (lnk, 2, buf2);
    recnumb = atol(buf2);
    dbpos (filea, recnuma);
    dbread (filea);
    dbputfld (filea, lnkflda, buf);
    dbpos (filea, recnuma);
    dbwrite (filea);
    dbpos (fileb, recnumb);
    dbread (fileb);
    dbputfld (fileb, lnkfldb, buf);
    dbpos (fileb, recnumb);
    dbwrite (fileb);

    if (grpflda)
    {
      // do group stuff
      dbgetfld (filea, grpflda, buf1);
      do
      {
        recnuma--;
        if (recnuma > 0)
        {
          dbpos (filea, recnuma);
          dbread (filea);
          dbgetfld (filea, grpflda, buf2);
        }
      } while (recnuma > 0 && strcmp (buf1, buf2) == 0);
      recnuma++;
      dbpos (filea, recnuma);
      dbread (filea);
      dbgetfld (filea, grpflda, buf2);
      while (recnuma <= data_a.nrecs && strcmp (buf1, buf2) == 0)
      {
        dbputfld (filea, lnkflda, buf);
        dbpos (filea, recnuma);
        dbwrite (filea);
        dbread (filea);
        recnuma++;
        dbgetfld (filea, grpflda, buf2);
      }
    }
    if (grpfldb)
    {
      // do group stuff
      dbgetfld (fileb, grpfldb, buf1);
      do
      {
        recnumb--;
        if (recnumb > 0)
        {
          dbpos (fileb, recnumb);
          dbread (fileb);
          dbgetfld (fileb, grpfldb, buf2);
        }
      } while (recnumb > 0 && strcmp (buf1, buf2) == 0);
      recnumb++;
      dbpos (fileb, recnumb);
      dbread (fileb);
      dbgetfld (fileb, grpfldb, buf2);
      while (recnumb <= data_b.nrecs && strcmp (buf1, buf2) == 0)
      {
        dbputfld (fileb, lnkfldb, buf);
        dbpos (fileb, recnumb);
        dbwrite (fileb);
        dbread (fileb);
        recnumb++;
        dbgetfld (fileb, grpfldb, buf2);
      }
    }
  }
  dbclose (lnk);
  dbclose (filea);
  dbclose (fileb);
}
