#include  <stdio.h>
#include  <stdlib.h>
#include  <string.h>

#include  "dblib.h"
#include  "compile.h"

static char copyright1[] = "(C)Board of Trustees of the University of Illinois, 1992";
static char copyright2[] = "Program conceived by Jeremy Atack and Fred Bateman";
static char copyright3[] = "Developed by Mary Eschelbach Gregson";
static char copyright4[] = "Programmed by Timothy Gregson";
static char copyright5[] = "Funded by the National Science Foundation under grants SES-8914861 and SES-9001066";

struct  tblfmt
{
  double  weight;
  int     used;
};

struct  fldlist
{
  int   flag;
  int   len;
};

void  main (int argc, char **argv);

void  main (int argc, char **argv)
{
  static  char      fld[256];

  FILE              *fp;
  int               i;
  struct  file_info finfo, datinfo[2];
  struct  fldinfo   infofld;
  struct  tblfmt    *cuttbl = NULL;
  int               file;
  int               dathand[2], lnkhand;
  long              recnum[2];
  double            cutoff = 0.0;
  int               cutflg = 0, process, lnkfld;
  double            weight;
  int               numsel, numblock, numfilter, nummatch;
  struct  fldlist   *flds;
  int               allflds = 0;
  struct  match_out *match;

  if (argc < 6)
  {
    fprintf (stderr, "Usage: rpt FILEA FILEB compfile linkfile cutoff <select by position in MATCH>\n");
    exit (1);
  }
  dbinit();
  if ((fp = fopen(argv[3], "rb")) == NULL)
  {
    fprintf (stderr, "Couldn't open compiled file:\n%s\n", argv[3]);
    exit (1);
  }
  fread (&numsel, 1, sizeof(int), fp);
  fread (&numblock, 1, sizeof(int), fp);
  fread (&numfilter, 1, sizeof(int), fp);
  fread (&nummatch, 1, sizeof(int), fp);
  match = malloc (sizeof (struct match_out) * nummatch);
  fseek (fp, (numsel * sizeof (struct select_out)) +
             (numblock * sizeof (struct block_out)) +
             (numfilter * sizeof (struct filter_out)) +
             8, SEEK_SET);
  fread (match, nummatch, sizeof (struct match_out), fp);
  fclose (fp);
  for (file = 0; file < 2; file++)
  {
    dathand[file] = dbopen (argv[file + 1], 0);
    dbgetinfo (dathand[file], &datinfo[file]);
  }
  lnkhand = dbopen (argv[4], 0);
  if (atof(argv[5]) != 0.0)
  {
    cutoff = atof (argv[4]);
    cutflg = 1;
    if ((datinfo[0].nrecs * sizeof (struct tblfmt)) > 65535)
    {
      fprintf (stderr, "Error allocating table\n");
      exit (10);
    }
    if ((cuttbl = malloc ((int)(datinfo[0].nrecs) * sizeof (struct tblfmt))) == NULL)
    {
      fprintf (stderr, "Error allocating table\n");
      exit (10);
    }
    memset (cuttbl, 0x0, sizeof (struct tblfmt) * (int)(datinfo[0].nrecs));
  }
  flds = malloc (sizeof (struct fldlist) * nummatch);
  memset (flds, 0x0, sizeof (struct fldlist) * nummatch);
  for (i = 0; i < nummatch; i++)
  {
    dbgetfldinfo (dathand[0], match[i].fldnuma, &infofld);
    flds[i].len = (infofld.length > 20 ? 20 : infofld.length);
    if (flds[i].len < 7)
      flds[i].len = 7;
  }
  if (argc > 6)
    for (i = 6; i < argc; i++)
      flds[atoi(argv[i]) - 1].flag = 1;
  else
    allflds = 1;
  dbgetinfo (lnkhand, &finfo);
//  printf ("%-11.11s %-41.41s %-41.41s %-3.3s %s\n", "Record #", "Last Name", "First Name", "MI", "Weight");
  while (dbread(lnkhand) == 0)
  {
    dbgetfld (lnkhand, (int)finfo.numflds, fld);
    weight = atof(fld);
    for (file = 0; file < 2; file++)
    {
      dbgetfld (lnkhand, file + 1, fld);
      recnum[file] = atol(fld);
    }
    if (cutflg && weight >= cutoff)
    {
      process = 0;
      if (!cuttbl[recnum[0]].used || (cuttbl[recnum[0]].weight == weight))
      {
        process = 1;
        cuttbl[recnum[0]].used = 1;
        cuttbl[recnum[0]].weight = weight;
      }
    }
    else
    {
      process = 1;
    }
    if (process)
    {
      for (file = 0; file < 2; file++)
      {
        dbpos (dathand[file], recnum[file]);
        dbread (dathand[file]);
        printf ("%4.4ld ", recnum[file]);
        for (i = 0; i < nummatch; i++)
        {
          if (flds[i].flag || (allflds && match[i].level == 1))
          {
            dbgetfld (dathand[file], (file == 0 ? match[i].fldnuma : match[i].fldnumb), fld);
            printf ("%-*.*s ", flds[i].len, flds[i].len, fld);
          }
        }
        printf ("\n");
      }
      for (i = 0, lnkfld = 0; i < nummatch; i++)
      {
        if (match[i].level == 1)
          lnkfld++;
        if (flds[i].flag || (allflds && match[i].level == 1))
        {
          dbgetfld (lnkhand, (lnkfld * 2) + 1, fld);
          printf ("%-*.*s ", flds[i].len, flds[i].len, fld);
        }
      }
      printf ("%lf\n\n", weight);
    }
  }
  if (cutflg)
    free (cuttbl);
  free (flds);
  dbclose (lnkhand);
  dbclose (dathand[0]);
  dbclose (dathand[1]);
}

void  pascal  freemem (void)
{
  // dummy routine for dblib to link correctly
}

