#include  "script.hxx"

static char copyright1[] = "(C)Board of Trustees of the University of Illinois, 1992";
static char copyright2[] = "Program conceived by Jeremy Atack and Fred Bateman";
static char copyright3[] = "Developed by Mary Eschelbach Gregson";
static char copyright4[] = "Programmed by Timothy Gregson";
static char copyright5[] = "Funded by the National Science Foundation under grants SES-8914861 and SES-9001066";

void main (int argc, char **argv);
struct  key_data  build_select (WINDOW *screen, WINDOW *build, struct fldinfo *fldlist[], struct file_info finfo[]);
struct  key_data  build_block (WINDOW *screen, WINDOW *build, struct fldinfo *fldlist[], struct file_info finfo[]);
struct  key_data  build_filter (WINDOW *screen, WINDOW *build, struct fldinfo *fldlist[], struct file_info finfo[]);
struct  key_data  build_match (WINDOW *screen, WINDOW *build, struct fldinfo *fldlist[], struct file_info finfo[]);
struct  key_data  build_submatch (WINDOW *screen, WINDOW *build, struct list_item *head, int *item_count);
int               exit_keys (WINDOW *wind, struct edit_line *start, long *home_pos, long line_count, struct key_data *inchar);

static  int       panic (struct key_data key);

FILE  *out;
WINDOW  *left, *right;

struct  type_list block_types[] =
  {
    {"NYSIIS", ""},
    {"SOUNDEX", ""},
    {"COPY", ""},
    {"FIRSTXLETTERS", "Enter number of characters to use"},
    {"", ""}
  };
struct  type_list filter_types[] =
  {
    {"NAME", "Enter number of chars to match"},
    {"NUMERIC", "Enter +/- range (single value)"},
    {"", ""}
  };
struct  type_list match_types[] =
  {
    {"NAME", ""},
    {"NUMERIC", "Enter any ranges (leave empty for exact only)"},
    {"TEXT", ""},
    {"", ""}
  };
struct  type_list submatch_types[] =
  {
    {"EXACT", ""},
    {"FIRSTXLETTERS", "Enter number of characters to use"},
    {"BLOCK", ""},
    {"", ""}
  };

void main (int argc, char **argv)
{
  int               dbhand[2];
  WINDOW            *screen, *build;
  short             oldrow, oldcol;
  struct  file_info finfo[2];
  struct  fldinfo   *fldlist[2];
  struct  key_data  inchar;
  int               file, i;
  char              *scriptfile;


  if (argc < 3 || argc > 4)
  {
    fprintf (stderr, "Usage: script FILEA FILEB <SCRIPTOUT>\n");
    exit (1);
  }
  for (file = 0; file < 2; file++)
  {
    if ((dbhand[file] = dbopen (argv[file+1], 0)) < 0)
    {
      fprintf (stderr, "Error opening dBase file: %s\n", argv[file+1]);
      exit (2);
    }
    dbgetinfo (dbhand[file], &finfo[file]);
    fldlist[file] = (struct fldinfo *)malloc ((int)finfo[file].numflds * sizeof(struct fldinfo));
    for (i = 0; i < (int)finfo[file].numflds; i++)
      dbgetfldinfo (dbhand[file], i+1, &fldlist[file][i]);
    dbclose (dbhand[file]);
  }
  if (argc == 4)
    scriptfile = argv[3];
  else
  {
    scriptfile = strdup (argv[1]);
    strcpy (&(scriptfile[strlen(scriptfile) - 3]), "SCR");
  }
  if ((out = fopen (scriptfile, "w")) == NULL)
  {
    fprintf (stderr, "Error opening output script file: %s\n", scriptfile);
    exit (3);
  }
  window_wrap = false;
  window_strip = false;
  get_cursor (&oldrow, &oldcol);
  pos_cursor (25, 1);                   // position cursor off screen
  install_int9 ();                      // install keyboard handler
  atexit (uninstall_int9);              // setup for uninstall
  add_hot_key (make_scrl, panic);       // allow break key during debugging
  screen = wopen (1, 1, 25, 80, sbcol, stcol, DOUBLE, 0, 1);
  wfillch (screen, 1, 1, 23, 78, 176);  // fill window with texture
  wheader (screen, "PC Matchmaker - Matching Instruction File Generation Program");
  left = wopen (4, 2, 9, 32, sbcol, stcol, 0, 0, 1);
  right = wopen (4, 39, 9, 69, sbcol, stcol, 0, 0, 1);
  wclear (left);
  wclear (right);
  wprintfat (screen, 1, 1, "File A: %s", argv[1]);
  wprintfat (screen, 1, 39, "File B: %s", argv[2]);
  wprintfat (screen, 2, 1, "Output: %s", scriptfile);

  build = wopen (11, 2, 24, 79, mbcol, mtcol, SINGLE, 0, 1);
  inchar = build_select (screen, build, fldlist, finfo);
  wclear (build);
  inchar = build_block (screen, build, fldlist, finfo);
  wclear (build);
  inchar = build_filter (screen, build, fldlist, finfo);
  wclear (build);
  inchar = build_match (screen, build, fldlist, finfo);
  wclose (&build);

  fclose (out);
  free (fldlist[0]);
  free (fldlist[1]);
  wclear (screen);
  wheader (screen, "Verify Matching Instruction File");
  wfooter (screen, "ESC to Abort  ENT to Continue");
  wview_ext = exit_keys;                // setup valid return keys
  inchar = wview (screen, scriptfile);
  wclose (&screen);                     // restore screen
  uninstall_int9 ();                    //    keyboard
  pos_cursor (oldrow, oldcol);          //    and cursor
  if (inchar.code != make_esc)
    exit (0);
  else
    exit (1);
}

struct  key_data  build_select (WINDOW *screen, WINDOW *build, struct fldinfo *fldlist[], struct file_info finfo[])
{
  struct  key_data  inchar;
  struct  list_item *head = NULL, *curr;
  int               pos[2], listhead;
  int               item_count;

  pos[0] = pos[1] = item_count = listhead = 0;
  wheader (build, "Building SELECT Section");
  curr = add_item (&head, 1);
  item_count++;
  do
  {
    wprintat (right, 1, 1, "Pick field from FILE A");
    wprintat (right, 2, 1, "(ESC for none  CTRL-ENT to End)");
    inchar = list_dbflds (4, 2, fldlist[0], (int)finfo[0].numflds, &pos[0]);
    wclear (right);
    if (!(inchar.code == make_ret && Control(inchar)))
    {
      if (inchar.code != make_esc)
      {
        memcpy (curr->fldname[0], fldlist[0][pos[0]].name, 10);
        disp_items (build, 75, &listhead, item_count, head);
        inchar = get_value (curr->data[0], 25, "Enter SELECT value for this field");
        if (inchar.code == make_esc)
        {
          memset (curr->fldname[0], 0x0, 10);
          memset (curr->data[0], 0x0, 25);
        }
        disp_items (build, 75, &listhead, item_count, head);
      }
      wprintat (left, 1, 1, "Pick field from FILE B");
      wprintat (left, 2, 1, "(ESC for none  CTRL-ENT to End)");
      inchar = list_dbflds (4, 40, fldlist[1], (int)finfo[1].numflds, &pos[1]);
      wclear (left);
      if (!(inchar.code == make_ret && Control(inchar)))
      {
        if (inchar.code != make_esc)
        {
          memcpy (curr->fldname[1], fldlist[1][pos[1]].name, 10);
          disp_items (build, 75, &listhead, item_count, head);
          inchar = get_value (curr->data[1], 25, "Enter SELECT value for this field");
          if (inchar.code == make_esc)
          {
            memset (curr->fldname[1], 0x0, 10);
            memset (curr->data[1], 0x0, 25);
          }
          disp_items (build, 75, &listhead, item_count, head);
        }
      }
    }
    if (curr->fldname[0][0] != 0 || curr->fldname[1][0] != 0)
    {
      curr = add_item (&head, 1);
      item_count++;
      listhead = (item_count < 12 ? 0 : item_count - 12);
    }
  } while (!(inchar.code == make_ret && Control(inchar)));
  item_count--;
  if (item_count)
    write_section (out, "SELECT", head, item_count);
  del_items (head);
  return (inchar);
}

struct  key_data  build_block (WINDOW *screen, WINDOW *build, struct fldinfo *fldlist[], struct file_info finfo[])
{
  struct  key_data  inchar;
  struct  list_item *head = NULL, *curr;
  int               pos[2], listhead;
  int               blocktype = 0;
  int               item_count;

  pos[0] = pos[1] = item_count = listhead = 0;
  wheader (build, "Building BLOCK Section");
  curr = add_item (&head, 2);
  item_count++;
  do
  {
    wprintat (right, 1, 1, "Pick BLOCK type");
    wprintat (right, 2, 1, "(CTRL-ENT to End)");
    do
      inchar = list_types (block_types, &blocktype);
    while (inchar.code != make_ret);
    wclear (right);
    if (!Control(inchar))
    {
      strcpy (curr->type, block_types[blocktype].type_name);
      disp_items (build, 75, &listhead, item_count, head);
      wprintat (right, 1, 1, "Pick field from FILE A");
      wprintat (right, 2, 1, "(ESC to Abort)");
      inchar = list_dbflds (4, 2, fldlist[0], (int)finfo[0].numflds, &pos[0]);
      wclear (right);
      if (inchar.code != make_esc)
      {
        memcpy (curr->fldname[0], fldlist[0][pos[0]].name, 10);
        disp_items (build, 75, &listhead, item_count, head);
        inchar = get_value (curr->data[0], 25, "Enter output field name");
        if (inchar.code != make_esc)
        {
          disp_items (build, 75, &listhead, item_count, head);
          wprintat (left, 1, 1, "Pick field from FILE B");
          wprintat (left, 2, 1, "(ESC for none  CTRL-ENT to End)");
          inchar = list_dbflds (4, 40, fldlist[1], (int)finfo[1].numflds, &pos[1]);
          wclear (left);
          if (inchar.code != make_esc)
          {
            memcpy (curr->fldname[1], fldlist[1][pos[1]].name, 10);
            disp_items (build, 75, &listhead, item_count, head);
            inchar = get_value (curr->data[1], 25, "Enter output field name");
            if (inchar.code != make_esc)
            {
              disp_items (build, 75, &listhead, item_count, head);
              if (block_types[blocktype].message[0] != 0)
              {
                inchar = get_value (curr->length, 3, block_types[blocktype].message);
              }
            }
          }
        }
      }
      if (inchar.code == make_esc)
      {
        item_count--;
        delete_item (&head, item_count);
        listhead = (item_count < 12 ? 0 : item_count - 12);
      }
      disp_items (build, 75, &listhead, item_count, head);
      curr = add_item (&head, 2);
      item_count++;
      listhead = (item_count < 12 ? 0 : item_count - 12);
    }
  } while (!Control(inchar));
  item_count--;
  if (item_count)
    write_section (out, "BLOCK", head, item_count);
  del_items (head);
  return (inchar);
}

struct  key_data  build_filter (WINDOW *screen, WINDOW *build, struct fldinfo *fldlist[], struct file_info finfo[])
{
  struct  key_data  inchar;
  struct  list_item *head = NULL, *curr;
  int               pos[2], listhead;
  int               filtertype = 0;
  int               item_count;

  pos[0] = pos[1] = item_count = listhead = 0;
  wheader (build, "Building FILTER Section");
  curr = add_item (&head, 3);
  item_count++;
  do
  {
    wprintat (right, 1, 1, "Pick FILTER type");
    wprintat (right, 2, 1, "(CTRL-ENT to End)");
    do
      inchar = list_types (filter_types, &filtertype);
    while (inchar.code != make_ret);
    wclear (right);
    if (!Control(inchar))
    {
      strcpy (curr->type, filter_types[filtertype].type_name);
      disp_items (build, 75, &listhead, item_count, head);
      wprintat (right, 1, 1, "Pick field from FILE A");
      wprintat (right, 2, 1, "(ESC to Abort)");
      inchar = list_dbflds (4, 2, fldlist[0], (int)finfo[0].numflds, &pos[0]);
      wclear (right);
      if (inchar.code != make_esc)
      {
        memcpy (curr->fldname[0], fldlist[0][pos[0]].name, 10);
        disp_items (build, 75, &listhead, item_count, head);
        wprintat (left, 1, 1, "Pick field from FILE B");
        wprintat (left, 2, 1, "(ESC for none  CTRL-ENT to End)");
        inchar = list_dbflds (4, 40, fldlist[1], (int)finfo[1].numflds, &pos[1]);
        wclear (left);
        if (inchar.code != make_esc)
        {
          memcpy (curr->fldname[1], fldlist[1][pos[1]].name, 10);
          disp_items (build, 75, &listhead, item_count, head);
          inchar = get_value (curr->data[0], 25, filter_types[filtertype].message);
          disp_items (build, 75, &listhead, item_count, head);
        }
      }
      if (inchar.code == make_esc)
      {
        item_count--;
        delete_item (&head, item_count);
        listhead = (item_count < 12 ? 0 : item_count - 12);
        disp_items (build, 75, &listhead, item_count, head);
      }
      curr = add_item (&head, 3);
      item_count++;
      listhead = (item_count < 12 ? 0 : item_count - 12);
    }
  } while (!Control(inchar));
  item_count--;
  if (item_count)
    write_section (out, "FILTER", head, item_count);
  del_items (head);
  return (inchar);
}

struct  key_data  build_match (WINDOW *screen, WINDOW *build, struct fldinfo *fldlist[], struct file_info finfo[])
{
  struct  key_data  inchar;
  struct  list_item *head = NULL, *curr;
  int               pos[2], listhead;
  int               matchtype = 0;
  int               item_count;

  pos[0] = pos[1] = item_count = listhead = 0;
  wheader (build, "Building MATCH Section");
  curr = add_item (&head, 3);
  item_count++;
  do
  {
    wprintat (right, 1, 1, "Pick MATCH type");
    wprintat (right, 2, 1, "(CTRL-ENT to End)");
    do
      inchar = list_types (match_types, &matchtype);
    while (inchar.code != make_ret);
    wclear (right);
    if (!Control(inchar))
    {
      strcpy (curr->type, match_types[matchtype].type_name);
      disp_items (build, 75, &listhead, item_count, head);
      wprintat (right, 1, 1, "Pick field from FILE A");
      wprintat (right, 2, 1, "(ESC to Abort)");
      inchar = list_dbflds (4, 2, fldlist[0], (int)finfo[0].numflds, &pos[0]);
      wclear (right);
      if (inchar.code != make_esc)
      {
        memcpy (curr->fldname[0], fldlist[0][pos[0]].name, 10);
        disp_items (build, 75, &listhead, item_count, head);
        wprintat (left, 1, 1, "Pick field from FILE B");
        wprintat (left, 2, 1, "(ESC for none  CTRL-ENT to End)");
        inchar = list_dbflds (4, 40, fldlist[1], (int)finfo[1].numflds, &pos[1]);
        wclear (left);
        if (inchar.code != make_esc)
        {
          memcpy (curr->fldname[1], fldlist[1][pos[1]].name, 10);
          disp_items (build, 75, &listhead, item_count, head);
          switch (matchtype)
          {
            case 0:   // name match
              inchar = build_submatch (screen, build, head, &item_count);
              if (inchar.code != make_esc)
              {
                listhead = (item_count < 12 ? 0 : item_count - 12);
                disp_items (build, 75, &listhead, item_count, head);
              }
              break;

            case 1:
              inchar = get_value (curr->data[0], 25, match_types[matchtype].message);
              if (inchar.code != make_esc)
              {
                disp_items (build, 75, &listhead, item_count, head);
              }
              break;
          }
        }
      }
      if (inchar.code == make_esc)
      {
        item_count--;
        delete_item (&head, item_count);
        listhead = (item_count < 12 ? 0 : item_count - 12);
        disp_items (build, 75, &listhead, item_count, head);
      }
      curr = add_item (&head, 3);
      item_count++;
      listhead = (item_count < 12 ? 0 : item_count - 12);
    }
    else
    {
      if (item_count - 1 == 0)
        inchar.code = 0;
    }
  } while (!(inchar.code == make_ret && Control(inchar)));
  item_count--;
  if (item_count)
    write_section (out, "MATCH", head, item_count);
  del_items (head);
  return (inchar);
}

struct  key_data  build_submatch (WINDOW *screen, WINDOW *build, struct list_item *head, int *item_count)
{
  struct  key_data  inchar;
  struct  list_item *curr;
  int               listhead;
  int               submatchtype = 0;
  int               start_item = *item_count;

  do
  {
    curr = add_item (&head, 4);
    (*item_count)++;
    listhead = (*item_count < 12 ? 0 : *item_count - 12);
    wprintat (right, 1, 1, "Pick MATCH subtype");
    wprintat (right, 2, 1, "(ESC to Abort  CTRL-ENT to End)");
    do
      inchar = list_types (submatch_types, &submatchtype);
    while (inchar.code != make_ret && inchar.code != make_esc);
    wclear (right);
    if (!Control(inchar) && inchar.code != make_esc)
    {
      strcpy (curr->type, submatch_types[submatchtype].type_name);
      disp_items (build, 75, &listhead, *item_count, head);
      if (submatch_types[submatchtype].message[0] != 0)
      {
        inchar = get_value (curr->length, 3, submatch_types[submatchtype].message);
        disp_items (build, 75, &listhead, *item_count, head);
      }
    }
  } while (!(inchar.code == make_ret && Control(inchar)) && !(inchar.code == make_esc));
  if (inchar.code == make_esc)
  {
    while (*item_count > (start_item + 1))
    {
      (*item_count)--;
      delete_item(&head, *item_count);
    }
  }
  else
    inchar.code = 0;
  (*item_count)--;
  delete_item (&head, *item_count);
  return (inchar);
}

int exit_keys (WINDOW *wind, struct edit_line *start, long *home_pos, long line_count, struct key_data *inchar)
{
  if (inchar->code == make_ret)
    return (1);
  else
    return (0);
}

#pragma check_stack (off)               // disable stack checks

static  int   panic (struct key_data key)
{
  if (Control(key))
  {
    outp (0x20, 0x20);                    // output end of interrupt to ctrlr
    outp (0x64, 0xae);                    // re-enable keyboard
    uninstall_int9();                     // uninstall keyboard handler
    cls (' ', 0x07);
    pos_cursor (0, 0);                    // position cursor back onto screen
    fcloseall ();
    _exit (255);                          // exit program
  }
  return (!0);                          // handle this key like normal
}

