#include  "scriputl.hxx"

static char copyright1[] = "(C)Board of Trustees of the University of Illinois, 1992";
static char copyright2[] = "Program conceived by Jeremy Atack and Fred Bateman";
static char copyright3[] = "Developed by Mary Eschelbach Gregson";
static char copyright4[] = "Programmed by Timothy Gregson";
static char copyright5[] = "Funded by the National Science Foundation under grants SES-8914861 and SES-9001066";

struct  list_item *add_item (struct list_item **head, char rec_type)
{
  struct  list_item *curr;

  if (*head == NULL)
    curr = *head = malloc (sizeof(struct list_item));
  else
  {
    for (curr = *head;curr->next != NULL;curr = curr->next)
    {
      // empty loop
    }
    curr->next = malloc (sizeof (struct list_item));
    curr = curr->next;
  }
  memset (curr, 0x0, sizeof (struct list_item));
  curr->rec_type = rec_type;
  return (curr);
}

struct  list_item *find_item(struct list_item *head, int index)
{
  struct list_item *curr;

  for (curr = head; curr && index; curr = curr->next, index--)
  {
    // empty loop
  }
  return (curr);
}

void  del_items (struct list_item *head)
{
  struct list_item  *curr;

  for (curr = head->next; head; curr = head->next)
  {
    free (head);
    head = curr;
  }
}

void  delete_item (struct list_item **head, int index)
{
  struct list_item  *curr, *temp;

  if (index == 0)
  {
    temp = *head;
    *head = (*head)->next;
    free (temp);
  }
  else
  {
    for (curr = *head; curr && index - 1; curr = curr->next, index--)
    {
      // empty loop
    }
    temp = curr->next;
    curr->next = temp->next;
    free (temp);
  }
  return;
}

void  disp_items (WINDOW            *wind,
                  short             col_width,
                  int               *head,
                  int               count,
                  struct list_item  *list)
{
  struct  list_item *curr;
  int   curpos;
  int   bord = (wind->border ? 1 : 0);
  int   num_rows = wind->row2 - wind->row1 - (bord * 2) + 1;
  int   num_cols = (wind->col2 - wind->col1 - (bord * 2) + 1)/col_width;
  int   disp_count = (num_rows * num_cols);
  int   row;
  int   col;
  char  ostrip = window_strip;

  window_strip = false;
  curr = find_item (list, *head);
  for (curpos = *head;curpos < count && curpos < *head + disp_count;curpos++)
  {
    row = ((curpos - *head) % num_rows) + 1;
    col = (((curpos - *head) / num_rows) * col_width) + 1;
    switch (curr->rec_type)
    {
      case  1:  // "  flda  fldb  val val"
        wprintfat (wind, row, col, "%-10.10s %-25.25s  %-10.10s %-25.25s",
          curr->fldname[0], curr->data[0], curr->fldname[1], curr->data[1]);
        break;

      case  2:  // "  type  flda  fldb  nflda nfldb [length]"
        wprintfat (wind, row, col, "%-13.13s %-10.10s %-10.10s %-10.10s %-10.10s",
          curr->type, curr->fldname[0], curr->data[0], curr->fldname[1],
          curr->data[1]);
        if (curr->length[0] != 0)
          wprintfat (wind, row, col+55, "%-4.4s", curr->length);
        break;

      case  3:  // "  type  flda  fldb  [val]"
        wprintfat (wind, row, col, "%-13.13s %-10.10s %-10.10s %-25.25s",
          curr->type, curr->fldname[0], curr->fldname[1], curr->data[0]);
        break;

      case  4:  // "    subtype [val]"
        wprintfat (wind, row, col+4, "%-13.13s", curr->type);
        if (curr->length[0] != 0)
          wprintfat (wind, row, col+4+14, "%s", curr->length);
        break;
    }
    curr = curr->next;
  }
  while (curpos < *head + disp_count)
  {
    row = ((curpos - *head) % num_rows) + 1;
    col = (((curpos - *head) / num_rows) * col_width) + 1;
    wprintfat (wind, row, col, "%*.*s", col_width, col_width, "");
    curpos++;
  }
  window_strip = ostrip;
}

struct  key_data  get_value (char *value, short len, char *prompt)
{
  WINDOW  *wind;
  struct  key_data  inchar;
  short   pos = 0, spos = 0;
  short   wlen = len + 2;

  len--;    // sinput doesn't count null terminator in length, we are
  value[0] = 0;
  if (len + 4 > 78)
    wlen = 76;
  if (wlen < strlen (prompt) + 4)
    wlen = strlen (prompt) + 4;
  if (wlen < 29)
    wlen = 29;
  wind = wopen (4, 2, 6, 2 + wlen, ebcol, etcol, DOUBLE, 0, 1);
  wheader (wind, prompt);
  wfooter (wind, "ESC to Abort  ENT to Accept");
  do
    inchar = sinput (5, 3, wlen - 2, len, value, &pos, &spos, UPPER);
  while (inchar.code != make_ret && inchar.code != make_esc);
  wclose (&wind);
  return (inchar);
}

void  disp_dbflds (WINDOW            *wind,
                   short             col_width,
                   int               *head,
                   int               count,
                   struct fldinfo    *list)
{
  int   curpos;
  int   bord = (wind->border ? 1 : 0);
  int   num_rows = wind->row2 - wind->row1 - (bord * 2) + 1;
  int   num_cols = (wind->col2 - wind->col1 - (bord * 2) + 1)/col_width;
  int   disp_count = (num_rows * num_cols);
  int   row;
  int   col;
  char  ostrip = window_strip;

  window_strip = false;
  for (curpos = *head;curpos < count && curpos < *head + disp_count;curpos++)
  {
    row = ((curpos - *head) % num_rows) + 1;
    col = (((curpos - *head) / num_rows) * col_width) + 1;
    wprintfat (wind, row, col, " %-10.10s  %c    %3d   %3d",list[curpos].name,
        list[curpos].type, list[curpos].length, list[curpos].decimals);
  }
  while (curpos < *head + disp_count)
  {
    row = ((curpos - *head) % num_rows) + 1;
    col = (((curpos - *head) / num_rows) * col_width) + 1;
    wprintfat (wind, row, col, "%*.*s", col_width, col_width, "");
    curpos++;
  }
  window_strip = ostrip;
}

struct  key_data  list_dbflds (short          start_row,
                               short          start_col,
                               struct fldinfo *fldlist,
                               int            numflds,
                               int            *pos)
{
  WINDOW            *wind;
  struct  key_data  inchar;
  int               head;

  wind = wopen (start_row, start_col, start_row + 5, start_col + 29, ebcol, etcol, SINGLE, 0, 1);
  wheader (wind, "Fld Name   Type Length Dec");
  wfooter (wind, "ENT - Select  ESC - Exit");
  head = -1;
  inchar.code = 0;
  do
  {
    inchar = wlist (wind, 27, pos, &head, numflds, disp_dbflds, whilite, NULL, fldlist);
  } while (inchar.code != make_esc && inchar.code != make_ret);
  wclose (&wind);
  return (inchar);
}

void  write_section (FILE *fp, char *header, struct list_item *list, short item_count)
{
  fprintf (fp, "%s\n", header);
  while (list && item_count)
  {
    trim (list->data[0], list->data[0]);
    trim (list->data[1], list->data[1]);
    trim (list->fldname[0], list->fldname[0]);
    trim (list->fldname[1], list->fldname[1]);
    trim (list->type, list->type);
    switch (list->rec_type)
    {
      case  1:  // "  flda  fldb  val val"
        fprintf (fp, "\t%s", (list->fldname[0][0] == 0 ? "NULL" : list->fldname[0]));
        fprintf (fp, "\t%s", (list->fldname[1][0] == 0 ? "NULL" : list->fldname[1]));
        fprintf (fp, "\t%s", (list->fldname[0][0] == 0 ? "0" : list->data[0]));
        fprintf (fp, "\t%s", (list->fldname[1][0] == 0 ? "0" : list->data[1]));
        fprintf (fp, "\n");
        break;

      case  2:  // "  type  flda  fldb  nflda nfldb [length]"
        fprintf (fp, "\t%s\t%s\t%s\t%s\t%s",
          list->type, list->fldname[0], list->fldname[1], list->data[0],
          list->data[1]);
        if (list->length[0] != 0)
          fprintf (fp, "\t%s", list->length);
        fprintf (fp, "\n");
        break;

      case  3:  // "  type  flda  fldb  [val]"
        fprintf (fp, "\t%s\t%s\t%s\t%s\n",
          list->type, list->fldname[0], list->fldname[1], list->data[0]);
        break;

      case  4:  // "    subtype [val]"
        fprintf (fp, "\t\t%s", list->type);
        if (list->length[0] != 0)
          fprintf (fp, "\t%s", list->length);
        fprintf (fp, "\n");
        break;
    }
    list = list->next;
    item_count--;
  }
  fprintf (fp, "\n");
}

struct  key_data  list_types (struct type_list  *list,
                              int               *pos)
{
  WINDOW            *wind;
  struct  key_data  inchar;
  int               head;
  int               count;

  for (count=0;list[count].type_name[0] != 0;count++);
  wind = wopen (4, 3, 9, 18, mbcol, mtcol, SINGLE, 0, 1);
  head = -1;
  inchar.code = 0;
  do
  {
    inchar = wlist (wind, 14, pos, &head, count, disp_types, whilite, NULL, list);
  } while (inchar.code != make_esc && inchar.code != make_ret);
  wclose (&wind);
  return (inchar);
}

void  disp_types (WINDOW            *wind,
                  short             col_width,
                  int               *head,
                  int               count,
                  struct type_list  *list)
{
  int   curpos;
  int   bord = (wind->border ? 1 : 0);
  int   num_rows = wind->row2 - wind->row1 - (bord * 2) + 1;
  int   num_cols = (wind->col2 - wind->col1 - (bord * 2) + 1)/col_width;
  int   disp_count = (num_rows * num_cols);
  int   row;
  int   col;
  char  ostrip = window_strip;

  window_strip = false;
  for (curpos = *head;curpos < count && curpos < *head + disp_count;curpos++)
  {
    row = ((curpos - *head) % num_rows) + 1;
    col = (((curpos - *head) / num_rows) * col_width) + 1;
    wprintfat (wind, row, col, " %-13.13s",list[curpos].type_name);
  }
  while (curpos < *head + disp_count)
  {
    row = ((curpos - *head) % num_rows) + 1;
    col = (((curpos - *head) / num_rows) * col_width) + 1;
    wprintfat (wind, row, col, "%*.*s", col_width, col_width, "");
    curpos++;
  }
  window_strip = ostrip;
}


